<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class AlteSalarioRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    
    public function getModalidadeSalarial($nrorg) {
        $sql = <<<SQL
    SELECT
        * 
    FROM
       FPA_TPMODALIDSAL 
    WHERE
       NRORG = :P_NRORG 
       OR NRORG = 0
SQL;

        
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }
    
    public function getTipoAlteraSal($nrorg) {
        $sql = <<<SQL
    SELECT
        * 
    FROM
       GPE_TIPOALTERA 
    WHERE
       NRORG = :P_NRORG 
       OR NRORG = 0
SQL;

        
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }
    
    public function getUltimoSalario($nrorg,$dtmescompetencia,$nrvinculom) {
        $sql = <<<SQL
   SELECT
       * 
    FROM
       GPE_ALTESALARIO ATS 
    WHERE
       ATS.NRORG = :P_NRORG 
       AND ATS.NRVINCULOM = :P_NRVINCULOM 
       AND ATS.DTALTESALARIO = 
       (
          SELECT
             MAX(DTALTESALARIO) 
          FROM
             GPE_ALTESALARIO 
          WHERE
             DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC) 
             AND NRORG = ATS.NRORG 
             AND NRVINCULOM = ATS.NRVINCULOM 
       )
SQL;

        
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_DTMESCOMPETENC'  => $dtmescompetencia,
                'P_NRVINCULOM'      => $nrvinculom
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }

    public function setAlteSalario(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO GPE_ALTESALARIO (
                NRALTESALARIO,      NRORG,                  NRVINCULOM,
                DTALTESALARIO,      NROCORRENCIA,           NRTIPOALTERA,
                VRSALARIO,          VRPERCALTERA,           VRPERCREAL,
                VRDIFERENCAREAL,   DTINCLUSAO,             DTULTATU,
                NRORGINCLUSAO,      CDOPERINCLUSAO,         NRORGULTATU,
                CDOPERULTATU,       DSOBSERVACAO,           NRTPMODALIDSAL,
                IDTPSALARIO,        DSVALSALARIOEXTENSO,    DTBASEDISSIDIO
            )
            VALUES (
                :NRALTESALARIO,     :NRORG,                 :NRVINCULOM,
                :DTALTESALARIO,     :NROCORRENCIA,          :NRTIPOALTERA,
                :VRSALARIO,         :VRPERCALTERA,          :VRPERCREAL,
                :VRDIFERENCAREAL,   :DTINCLUSAO,            :DTULTATU,
                :NRORGINCLUSAO,     :CDOPERINCLUSAO,        :NRORGULTATU,
                :CDOPERULTATU,      :DSOBSERVACAO,          :NRTPMODALIDSAL,
                :IDTPSALARIO,       :DSVALSALARIOEXTENSO,   :DTBASEDISSIDIO
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getAltesalario(
        array $params
    ) {
        $sql = <<<SQL
        SELECT
            ALTE.NRORG,
            ALTE.NRALTESALARIO,
            ALTE.NRVINCULOM,
            ALTE.NRTPMODALIDSAL,
            TO_CHAR(ALTE.DTALTESALARIO, 'DD/MM/YYYY') AS DTALTESALARIO,
            ALTE.DTALTESALARIO AS DTALTESALARIO_ORDER,
            ALTE.NRTIPOALTERA,
            TO_CHAR(ALTE.DTBASEDISSIDIO, 'DD/MM/YYYY') AS DTBASEDISSIDIO,
            ALTE.NROCORRENCIA,
            ALTE.IDTPSALARIO,
            TO_CHAR(ALTE.VRSALARIO, '999G999G990D00', 'NLS_NUMERIC_CHARACTERS = ,.') AS VRSALARIO,
            TO_CHAR(ALTE.VRSALARIOHR, '999G999G990D00', 'NLS_NUMERIC_CHARACTERS = ,.') AS VRSALARIOHR,
            ALTE.DSOBSERVACAO,
            ALTE.VRPERCALTERA,
            ALTE.NRORGINCLUSAO,
            ALTE.DTINCLUSAO,
            ALTE.CDOPERINCLUSAO,
            ALTE.NRORGULTATU,
            ALTE.DTULTATU,
            ALTE.CDOPERULTATU,
            ALTE.DSVALSALARIOEXTENSO,
            ALTE.NMVINCULOM,
            ALTE.QTHRSEMESCTRABH,
            ALTE.NRESTRUTLEGAL,
            ALTE.NMESTRUTLEGAL,
            ALTE.NRESTRUTGEREN,
            ALTE.NMESTRUTGEREN,
            TO_CHAR(ALTE.VRSALARIO_ANTE, '999G999G990D00', 'NLS_NUMERIC_CHARACTERS = ,.') AS VRSALARIO_ANTE,
            ALTE.NMTIPOALTERA,
            ALTE.NMTPMODALIDSAL,
            ALTE.DSTIPOSALARIO,
            ALTE.NRTIPOVINCULOM,
            TO_CHAR(ROUND(NVL(VRDIFERENCAREAL, DECODE(VRSALARIO_ANTE, NULL, NULL, VRSALARIO - VRSALARIO_ANTE)), 2), '999G999G990D00', 'NLS_NUMERIC_CHARACTERS = ,.') AS VRDIFERENCAREAL,
            TO_CHAR(ROUND(NVL(VRPERCREAL, DECODE(VRSALARIO_ANTE, NULL, NULL,((VRSALARIO / VRSALARIO_ANTE) - 1) * 100)), 2), '999G999G990D00', 'NLS_NUMERIC_CHARACTERS = ,.') AS VRPERCREAL
        FROM
            (
                SELECT
                    GPE_ALTESALARIO.*,
                    VINC.NMVINCULOM,
                    VINC.NRTIPOVINCULOM,
                    VINC.NRESTRUTGEREN,
                    VINC.NRESTRUTLEGAL,
                    ESCH.QTHRSEMESCTRABH,
                    EHG.NMESTRUTURAH   NMESTRUTGEREN,
                    EHL.NMESTRUTURAH   NMESTRUTLEGAL,
                    LEAD(GPE_ALTESALARIO.VRSALARIO, 1) OVER(
                        PARTITION BY GPE_ALTESALARIO.NRVINCULOM
                        ORDER BY
                            DTALTESALARIO DESC, NRALTESALARIO DESC
                    ) VRSALARIO_ANTE,
                    GPE_TIPOALTERA.NMTIPOALTERA,
                    FPA_TPMODALIDSAL.NMTPMODALIDSAL,
                    GPE_TIPOSALARIO.DSTIPOSALARIO
                FROM
                    GPE_ALTESALARIO
                    JOIN (
                        SELECT
                            GPE_VINCULOM.NRORG,
                            GPE_VINCULOM.NRVINCULOM,
                            GPE_VINCULOM.NMVINCULOM,
                            GPE_VINCULOM.CDMATRICULA,
                            VH.NRESTRUTGEREN,
                            VH.NRESTRUTLEGAL,
                            VH.NRESCALATRABM,
                            GPE_VINCULOM.NRTIPOVINCULOM
                        FROM
                            GPE_VINCULOM
                            JOIN GPE_VINCULOH VH ON VH.NRORG = GPE_VINCULOM.NRORG
                                                    AND VH.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                                                    AND VH.DTMESCOMPETENC = (
                                SELECT
                                    MAX(DTMESCOMPETENC)
                                FROM
                                    GPE_VINCULOH
                                WHERE
                                    NRORG = GPE_VINCULOM.NRORG
                                    AND NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                                    AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
                            )
                        WHERE
                            ( GPE_VINCULOM.DTRESCISAOVINC IS NULL
                              OR GPE_VINCULOM.DTRESCISAOVINC >= :P_DTMESCOMPETENC
                              OR NVL(:EXISTE_VINCULOS, 'N') != 'S' )
                        ORDER BY
                            GPE_VINCULOM.NMVINCULOM
                    ) VINC ON VINC.NRVINCULOM = GPE_ALTESALARIO.NRVINCULOM
                              AND VINC.NRORG = GPE_ALTESALARIO.NRORG
                    JOIN GPE_ESCALATRABH                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         ESCH ON ESCH.NRORG = VINC.NRORG
                                                 AND ESCH.NRESCALATRABM = VINC.NRESCALATRABM
                                                 AND ESCH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            DTMESCOMPETENC <= :P_DTMESCOMPETENC
                            AND NRORG = ESCH.NRORG
                            AND NRESCALATRABM = ESCH.NRESCALATRABM
                    )
                    LEFT JOIN ESTRUTURAH                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              EHG ON EHG.NRORG = VINC.NRORG
                                                AND EHG.NRESTRUTURAM = VINC.NRESTRUTGEREN
                                                AND EHG.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH
                        WHERE
                            DTMESCOMPETENC <= :P_DTMESCOMPETENC
                            AND NRORG = EHG.NRORG
                            AND NRESTRUTURAM = EHG.NRESTRUTURAM
                    )
                    LEFT JOIN ESTRUTURAH                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              EHL ON EHL.NRORG = VINC.NRORG
                                                AND EHL.NRESTRUTURAM = VINC.NRESTRUTLEGAL
                                                AND EHL.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH
                        WHERE
                            DTMESCOMPETENC <= :P_DTMESCOMPETENC
                            AND NRORG = EHL.NRORG
                            AND NRESTRUTURAM = EHL.NRESTRUTURAM
                    )
                    JOIN GPE_TIPOALTERA ON GPE_TIPOALTERA.NRTIPOALTERA = GPE_ALTESALARIO.NRTIPOALTERA
                                           AND ( GPE_TIPOALTERA.NRORG = GPE_ALTESALARIO.NRORG
                                                 OR GPE_TIPOALTERA.NRORG = :P_NRORG_PADRAO )
                    JOIN FPA_TPMODALIDSAL ON FPA_TPMODALIDSAL.NRTPMODALIDSAL = GPE_ALTESALARIO.NRTPMODALIDSAL
                                             AND ( FPA_TPMODALIDSAL.NRORG = GPE_ALTESALARIO.NRORG
                                                   OR FPA_TPMODALIDSAL.NRORG = :P_NRORG_PADRAO )
                    JOIN GPE_TIPOSALARIO ON GPE_TIPOSALARIO.CDTIPOSALARIO = GPE_ALTESALARIO.IDTPSALARIO
                                            AND ( GPE_TIPOSALARIO.NRORG = GPE_ALTESALARIO.NRORG
                                                  OR GPE_TIPOSALARIO.NRORG = :P_NRORG_PADRAO )
                WHERE
                    GPE_ALTESALARIO.NRORG = :P_NRORG
                    AND (GPE_ALTESALARIO.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                    AND GPE_ALTESALARIO.VRSALARIO >= 0
            ) ALTE
        ORDER BY NRVINCULOM, DTALTESALARIO_ORDER DESC
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute($params);

        return $query->fetchAll();
    }
}