<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Ems\Services\DocumentoService;
use Ems\Util\Utils;
use Util\NovoCodigo;

class DocumentoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function get(
        int $nrOrg,
        int $nrAnexo
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GPE_ANEXOS
            WHERE
                NRORG = :P_NRORG AND
                NRANEXO = :P_NRANEXO
SQL;

        $params = [
            'P_NRORG'   => $nrOrg,
            'P_NRANEXO' => $nrAnexo,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $documento = $std->fetch();

        if ($documento === false) {
            return []; 
        }

        return $documento;
    }

    public function getDocumentosVinculo(
        array $params
    ): array {
        $params['P_NRCATEGORIA'] =
            $params['P_NRCATEGORIA'] ??
            null;

        $sql = <<<SQL
            SELECT
                ANEX.*,
                CAT.NMCATEGDOCUMENTO,
                TIPO.NMTIPODOCUMENTO
            FROM
                GPE_VINCULOM VM
                JOIN
                GPE_PESSOA P ON
                    P.NRORG = VM.NRORG AND
                    P.NRPESSOA = VM.NRPESSOA
                JOIN
                GPE_ANEXOS ANEX ON
                    ANEX.NRORG = P.NRORG AND
                    ANEX.NRPARCNEGOCIO = P.NRPARCNEGOCIO
                LEFT JOIN
                GER_CATEGDOCUMENTO CAT ON
                    CAT.NRORG = ANEX.NRORG AND
                    CAT.NRCATEGDOCUMENTO = ANEX.NRCATEGDOCUMENTO
                LEFT JOIN
                GER_TIPODOCUMENTO TIPO ON
                    TIPO.NRORG = ANEX.NRORG AND
                    TIPO.NRTIPODOCUMENTO = ANEX.NRTIPODOCUMENTO
            WHERE
                VM.NRORG = :P_NRORG AND
                VM.NRVINCULOM = :P_NRVINCULO AND
                (
                    (
                        :P_NRCATEGORIA IS NOT NULL AND
                        ANEX.NRCATEGDOCUMENTO = :P_NRCATEGORIA
                    ) OR
                    (
                        :P_NRCATEGORIA = 'null' AND
                        ANEX.NRCATEGDOCUMENTO IS NULL
                    ) OR
                    :P_NRCATEGORIA IS NULL
                ) AND
                (
                    (
                        :P_NOMECATEG IS NOT NULL AND
                        UPPER(CAT.NMCATEGDOCUMENTO) LIKE UPPER(:P_NOMECATEG)
                    ) OR
                    :P_NOMECATEG IS NULL
                ) AND
                (
                    (
                        :P_NOMETIPO IS NOT NULL AND
                        UPPER(TIPO.NMTIPODOCUMENTO) LIKE UPPER(:P_NOMETIPO)
                    ) OR
                    :P_NOMETIPO IS NULL
                ) AND
                TIPO.IDDOCUMENTOPUBLICO = 'S'
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $documentos = $std->fetchAll();

        return $documentos;
    }

    public function getCategoriasComDocumentoVinculo(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                CAT.NRCATEGDOCUMENTO,
                CAT.NMCATEGDOCUMENTO
            FROM
                GPE_VINCULOM VM
                JOIN
                GPE_PESSOA P ON
                    P.NRORG = VM.NRORG AND
                    P.NRPESSOA = VM.NRPESSOA
                JOIN
                GPE_ANEXOS ANEX ON
                    ANEX.NRORG = P.NRORG AND
                    ANEX.NRPARCNEGOCIO = P.NRPARCNEGOCIO
                LEFT JOIN
                GER_CATEGDOCUMENTO CAT ON
                    CAT.NRORG = ANEX.NRORG AND
                    CAT.NRCATEGDOCUMENTO = ANEX.NRCATEGDOCUMENTO
                LEFT JOIN
                GER_TIPODOCUMENTO TIPO ON
                    TIPO.NRORG = ANEX.NRORG AND
                    TIPO.NRTIPODOCUMENTO = ANEX.NRTIPODOCUMENTO
            WHERE
                VM.NRORG = :P_NRORG AND
                VM.NRVINCULOM = :P_NRVINCULO AND
                TIPO.IDDOCUMENTOPUBLICO = 'S'
            GROUP BY
                CAT.NRCATEGDOCUMENTO,
                CAT.NMCATEGDOCUMENTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $categorias = $std->fetchAll();

        return $categorias;
    }

    public function getDoumentoQrCodeVinculo(
        int $nrParcNegocio
    ) {
        $sql = <<<SQL
            SELECT
                CAT.NMCATEGDOCUMENTO,
                TIPO.NMTIPODOCUMENTO,
                AN.NRANEXO,
                AN.NRORG,
                AN.NRPARCNEGOCIO,
                AN.NMANEXO,
                AN.DSREFERENCIA,
                AN.IDTYPE,
                AN.DSURL,
                AN.DTINCLUSAO,
                AN.DTULTATU,
                AN.CDOPERINCLUSAO,
                AN.CDOPERULTATU,
                AN.IDFOTO,
                AN.NRCATEGDOCUMENTO,
                AN.NRTIPODOCUMENTO,
                AN.IDDOCAPROVADO
            FROM GPE_ANEXOS AN
            LEFT JOIN GER_CATEGDOCUMENTO CAT ON
                CAT.NRORG = AN.NRORG AND
                CAT.NRCATEGDOCUMENTO = AN.NRCATEGDOCUMENTO
            LEFT JOIN GER_TIPODOCUMENTO TIPO ON
                TIPO.NRORG = AN.NRORG AND
                TIPO.NRTIPODOCUMENTO = AN.NRTIPODOCUMENTO
            WHERE
                (AN.NRORG = :P_NRORG) AND
                AN.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
                AN.NRTIPODOCUMENTO = '1'

SQL;

        $params = [
            'P_NRORG'         => Utils::getParamters()->getNrOrg(),
            'P_NRPARCNEGOCIO' => $nrParcNegocio
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $docQrCode = $std->fetch();

        return $docQrCode;
    }

    public function updateFile(
        int $nrAnexo,
        string $filePath
    ) {
        $sql = <<<SQL
            UPDATE
                GPE_ANEXOS
            SET
                DSPATH = :P_DSPATH,
                CDOPERULTATU = :P_CDOPERADOR,
                DTULTATU = SYSDATE
            WHERE
                NRORG = :P_NRORG AND
                NRANEXO = :P_NRANEXO
SQL;

        $params = [
            'P_NRORG' => Utils::getParamters()->getNrOrg(),
            'P_CDOPERADOR' => Utils::getParamters()->getCdOperador(),
            'P_NRANEXO' => $nrAnexo,
            'P_DSPATH' => $filePath
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getCategorias(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GER_CATEGDOCUMENTO
            WHERE
                IDATIVO = 'S' AND
                NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $categorias = $std->fetchAll();

        return $categorias;
    }

    public function getCategEspecifica(
        string $nomeCateg
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GER_CATEGDOCUMENTO
            WHERE
                IDATIVO = 'S' AND
                NRORG = :P_NRORG AND
                UPPER(NMCATEGDOCUMENTO) = UPPER(:P_NOMECATEG)
SQL;

        $params = [
            'P_NRORG' => Utils::getParamters()->getNrOrg(),
            'P_NOMECATEG' => $nomeCateg,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $categoria = $std->fetchAll();

        return empty($categoria) ? [] : reset($categoria);
    }

    public function getTipoEspecifico(
        string $nomeTipo
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GER_TIPODOCUMENTO
            WHERE
                IDATIVO = 'S' AND
                NRORG = :P_NRORG AND
                UPPER(NMTIPODOCUMENTO) = UPPER(:P_NOMETIPO)
SQL;

        $params = [
            'P_NRORG' => Utils::getParamters()->getNrOrg(),
            'P_NOMETIPO' => $nomeTipo,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipo = $std->fetchAll();

        return empty($tipo) ? [] : reset($tipo);
    }

    public function criaCategEspecifica(string $nomeCateg): bool {
        $sql = <<<SQL
            INSERT INTO GER_CATEGDOCUMENTO
            (
                NRORG, NRCATEGDOCUMENTO, NMCATEGDOCUMENTO,
                IDATIVO, DTINCLUSAO, DTULTATU,
                NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU
            )
            VALUES
            (
                :P_NRORG, :P_NRCATEGORIA, :P_NOME_ESPEC,
                'S', SYSDATE, NULL,
                :P_NRORG, :P_CDOPERADOR, NULL, NULL
            )
SQL;

        $params = [
            'P_NRORG' => Utils::getParamters()->getNrOrg(),
            'P_NRCATEGORIA' => NovoCodigo::geraCodigo(Utils::getParamters()->getNrOrg(), 'GER_CATEGDOCUMENTO', 1, 12, true),
            'P_NOME_ESPEC' => $nomeCateg,
            'P_CDOPERADOR' => Utils::getParamters()->getCdOperador(),

        ];

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }

    public function criaTipoEspecifico(
        string $nomeTipo,
        ?string $sigla = null,
        ?string $publico = DocumentoService::DEFAULT_DOCUMENTO_PUBLICO
    ): bool {
        $sql = <<<SQL
            INSERT INTO GER_TIPODOCUMENTO
            (
                NRORG, NRTIPODOCUMENTO, NMTIPODOCUMENTO,
                DSSIGLADOCUMENTO, IDDOCUMENTOPUBLICO, IDATIVO,
                DTINCLUSAO, DTULTATU, NRORGINCLUSAO,
                CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU
            )
            VALUES
            (
                :P_NRORG, :P_NRTIPO, :P_NOME_ESPEC,
                :P_SIGLA, :P_PUBLICO, 'S',
                SYSDATE, NULL, :P_NRORG,
                :P_CDOPERADOR, NULL, NULL
            )
SQL;

        $params = [
            'P_NRORG' => Utils::getParamters()->getNrOrg(),
            'P_NRTIPO' => NovoCodigo::geraCodigo(Utils::getParamters()->getNrOrg(), 'GER_CATEGDOCUMENTO', 1, 12, true),
            'P_NOME_ESPEC' => $nomeTipo,
            'P_SIGLA' => $sigla,
            'P_PUBLICO' => $publico,
            'P_CDOPERADOR' => Utils::getParamters()->getCdOperador(),

        ];

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }

    public function getTipos(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GER_TIPODOCUMENTO
            WHERE
                IDATIVO = 'S' AND
                NRORG = :P_NRORG AND
                IDDOCUMENTOPUBLICO = 'S'
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipos = $std->fetchAll();

        return $tipos;
    }

    public function saveDocumento(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO GPE_ANEXOS
            (
                NRANEXO,            NRORG,              NRPARCNEGOCIO,
                NMANEXO,            DSREFERENCIA,       IDTYPE,
                DSPATH,             DTINCLUSAO,         CDOPERINCLUSAO,
                IDFOTO,             NRCATEGDOCUMENTO,   NRTIPODOCUMENTO
            )
            VALUES
            (
                :NRANEXO,           :NRORG,             :NRPARC,
                :NMANEXO,           :DSREFERENCIA,      :IDTYPE,
                :DSPATH,            SYSDATE,            :CDOPERINCLUSAO,
                :IDFOTO,            :NRCATEGORIA,       :NRTIPO
            )
SQL;

        $params['NMANEXO']          = $params['NMANEXO'] ?? $params['DSREFERENCIA'];
        $params['IDTYPE']           = $params['IDTYPE'] ?? 'image/jpeg';
        $params['CDOPERINCLUSAO']   = $params['CDOPERINCLUSAO'] ?? 1;
        $params['IDFOTO']           = $params['IDFOTO'] ?? 'S';

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function deleteDocumento(int $nrAnexo): bool {
        $sql = <<<SQL
            DELETE FROM GPE_ANEXOS
            WHERE NRANEXO = :P_NRANEXO
              AND NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRORG' => Utils::getParamters()->getNrOrg(),
            'P_NRANEXO' => $nrAnexo
        ];

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }

    public function getDadosCategoria(
        int $nrOrg,
        int $nrCategoria
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GER_CATEGDOCUMENTO
            WHERE
                NRORG = :P_NRORG AND
                NRCATEGDOCUMENTO = :P_NRCATEGORIA
SQL;

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRCATEGORIA' => $nrCategoria,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $categoria = $std->fetch();

        return $categoria;
    }

    public function updateDocumentoAssinatura(
        array $params
    ) {
        $sql = <<<SQL
        UPDATE GPE_ANEXOS GPA
        SET
            DSPATH = :DSPATH,
            IDTYPE = :IDTYPE,
            DTULTATU = SYSDATE,
            CDOPERULTATU = :CDOPERADOR
        WHERE 
            GPA.NRORG = :NRORG AND
            GPA.NRPARCNEGOCIO = :NRPARCNEGOCIO AND
            GPA.NRANEXO = :NRANEXO
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function getDadosTipo(
        int $nrOrg,
        int $nrTipo
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GER_TIPODOCUMENTO
            WHERE
                NRORG = :P_NRORG AND
                NRTIPODOCUMENTO = :P_NRTIPO
SQL;

        $params = [
            'P_NRORG'   => $nrOrg,
            'P_NRTIPO'  => $nrTipo,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipo = $std->fetch();

        return $tipo;
    }
    public function getDoumentoByCategory(
        int $nrParcNegocio,
        int $nrCategDocumento
    ) {
        $sql = <<<SQL
            SELECT
                CAT.NMCATEGDOCUMENTO,
                AN.NRANEXO,
                AN.NRORG,
                AN.NRPARCNEGOCIO,
                AN.NMANEXO,
                AN.DSREFERENCIA,
                AN.IDTYPE,
                AN.DSURL,
                AN.DTINCLUSAO,
                AN.DTULTATU,
                AN.CDOPERINCLUSAO,
                AN.CDOPERULTATU,
                AN.IDFOTO,
                AN.NRCATEGDOCUMENTO,
                AN.NRTIPODOCUMENTO,
                AN.IDDOCAPROVADO
            FROM GPE_ANEXOS AN
            LEFT JOIN GER_CATEGDOCUMENTO CAT ON
                CAT.NRORG = AN.NRORG AND
                CAT.NRCATEGDOCUMENTO = AN.NRCATEGDOCUMENTO
            WHERE
                (AN.NRORG = :P_NRORG) AND
                AN.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
                AN.NRCATEGDOCUMENTO = :P_NRCATEGDOCUMENTO

SQL;

        $params = [
            'P_NRORG'            => Utils::getParamters()->getNrOrg(),
            'P_NRPARCNEGOCIO'    => $nrParcNegocio,
            'P_NRCATEGDOCUMENTO' => $nrCategDocumento
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $docQrCode = $std->fetch();

        return $docQrCode;
    }
}