<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class MovimentacaoRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getUltimaMovimentacaoLegal($nrorg, $nrvinculom) {
        $sql = <<<SQL
            SELECT
                MVT.NRESTRUTURAM,
                MVT.NRTPMOVTRANSFM,
                MVT.NRMOTIVOTRANSF
                FROM
                    GPE_MOVIMENTACAO MVT 
                WHERE
                    MVT.NRORG = :P_NRORG AND
                    MVT.NRVINCULOM = :P_NRVINCULOM AND
                    MVT.NRTIPOESTRUTURA = (
                            SELECT NRTPESTRUTLEGAL
                            FROM GPE_PARAMORG
                            WHERE NRORG = MVT.NRORG
                    ) AND
                    MVT.DTINIMOVIMENT = (
                            (SELECT DTINIMOVIMENT FROM 
                                (SELECT 
                                    MVTT.DTINIMOVIMENT 
                                FROM 
                                    GPE_MOVIMENTACAO MVTT 
                                WHERE 
                                    MVTT.NRORG = MVT.NRORG AND
                                    MVTT.NRVINCULOM = MVT.NRVINCULOM AND
                                    MVTT.NRTIPOESTRUTURA = MVT.NRTIPOESTRUTURA
                                ORDER BY MVTT.DTINIMOVIMENT DESC) 
                            WHERE ROWNUM = 1)
                        )
SQL;

        
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom
            )
        );

        $resultQuery = $query->fetch();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }

    public function getUltimaMovimentacaoGeren($nrorg, $nrvinculom) {
        $sql = <<<SQL
            SELECT
                MVT.NRESTRUTURAM,
                MVT.NRTPMOVTRANSFM,
                MVT.NRMOTIVOTRANSF
                FROM
                    GPE_MOVIMENTACAO MVT 
                WHERE
                    MVT.NRORG = :P_NRORG AND
                    MVT.NRVINCULOM = :P_NRVINCULOM AND
                    MVT.NRTIPOESTRUTURA = (
                            SELECT NRTPESTRUTGEREN
                            FROM GPE_PARAMORG
                            WHERE NRORG = MVT.NRORG
                    ) AND
                    MVT.DTINIMOVIMENT = (
                            (SELECT DTINIMOVIMENT FROM 
                                (SELECT 
                                    MVTT.DTINIMOVIMENT 
                                FROM 
                                    GPE_MOVIMENTACAO MVTT 
                                WHERE 
                                    MVTT.NRORG = MVT.NRORG AND
                                    MVTT.NRVINCULOM = MVT.NRVINCULOM AND
                                    MVTT.NRTIPOESTRUTURA = MVT.NRTIPOESTRUTURA
                                ORDER BY MVTT.DTINIMOVIMENT DESC) 
                            WHERE ROWNUM = 1)
                        )
SQL;

        
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom
            )
        );

        $resultQuery = $query->fetch();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }

    public function setMovimentacao(
        array $params
    ): void {
        $sql = <<< SQL
            INSERT INTO GPE_MOVIMENTACAO (
                NRMOVIMENTACAO,     NRORG,              NRVINCULOM,
                NRTIPOTRANSFER,     NRTPMOVTRANSFM,     NRMOTIVOTRANSF,
                NRESTRUTURAM,       DTINIMOVIMENT,      DTFIMMOVIMENT,
                DSOBSERVACAO,       DTINCLUSAO,         DTULTATU,
                NRORGINCLUSAO,      CDOPERINCLUSAO,     NRORGULTATU,
                CDOPERULTATU,       NRTIPOESTRUTURA,    DTMOVIMENTRETRO,
                IDATUALIZAACERTO,   NRMOTIVOAFASTA
            )
            VALUES (
                :NRMOVIMENTACAO,    :NRORG,             :NRVINCULOM,
                :NRTIPOTRANSFER,    :NRTPMOVTRANSFM,    :NRMOTIVOTRANSF,
                :NRESTRUTURAM,      :DTINIMOVIMENT,     :DTFIMMOVIMENT,
                :DSOBSERVACAO,      :DTINCLUSAO,        :DTULTATU,
                :NRORGINCLUSAO,     :CDOPERINCLUSAO,    :NRORGULTATU,
                :CDOPERULTATU,      :NRTIPOESTRUTURA,   :DTMOVIMENTRETRO,
                :IDATUALIZAACERTO,  :NRMOTIVOAFASTA
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}