<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Util\{ NovoCodigo };

class SolicitacaoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getSolicitacoesVinculosGestor(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                SL.NRORG,
                SL.NRSOLICITACAO,
                SL.NRTIPOSOLICITACAO,
                SL.NRVINCULOM,
                TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY') AS DTSOLICITACAO,
                SL.IDAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOPRI,  'A', 'Aprovada',
                                            'R', 'Reprovada',
                                            'P', 'Pendente',
                                            'E', 'Em andamento',
                                            'Pendente') DSAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOSEC,  'A', 'Aprovada',
                                            'R', 'Reprovada',
                                            'P', 'Pendente',
                                            'E', 'Em andamento',
                                            'Pendente') DSAPROVACAOSEC,
                SL.DTAPROVACAOPRI,
                SL.CDOPERAPROVPRI,
                SL.IDAPROVACAOSEC,
                SL.DTAPROVACAOSEC,
                SL.CDOPERAPROVSEC,
                SL.DSOBSSOLICITACAO,
                SL.DSOBSAPROVAPRI,
                SL.DSOBSAPROVASEC,
                SL.DSPATH,
                VM_S.NMVINCULOM,
                OP.CDOPERADOR,
                OP.NMOPERADOR,
                TP.NMTIPOSOLICITACAO,
                FASE_SEGUINTE.NRFASEAPROVSOLI,
                FASE_SEGUINTE.NRORDEM
                    || '-'
                    || FASE_SEGUINTE.NMFASEAPROVSOLI AS NMFASEAPROVSOLI,
                CASE
                    WHEN :P_NRPERFAPROVASOLI IS NOT NULL AND SL.IDAPROVACAOSEC NOT IN ('A', 'R') THEN 'S'
                    WHEN EXISTS (
                        SELECT 1
                        FROM PERFACESSPARC PERACESS_APROV
                        WHERE PERACESS_APROV.NRORG = SL.NRORG
                            AND INSTR(',' || FASE_SEGUINTE.DSNRPERFILACESSO || ',', ',' || PERACESS_APROV.NRPERFILACESSO ||  ',') > 0
                            AND PERACESS_APROV.NRPARCNEGOCIO = (
                                SELECT NRPARCNEGOCIO
                                FROM OPERADOR
                                WHERE NRORG = PERACESS_APROV.NRORG AND CDOPERADOR = :P_CDOPERADOR_APROVANTE
                            )
                    ) AND SL.IDAPROVACAOSEC NOT IN ('A', 'R') THEN 'S'
                    ELSE 'N'
                END ID_OPER_APROVA_SOLI,
                CASE
                    WHEN FASE_SEGUINTE_PROXIMA.NRFASEAPROVSOLI IS NOT NULL AND SL.IDAPROVACAOSEC NOT IN ('A', 'R') THEN 'S'
                    ELSE 'N'
                END ID_FASE_SEGUINTE_EXISTS,
                FASE_SEGUINTE_PROXIMA.NRFASEAPROVSOLI AS NRFASEAPROVSOLI_SEGUINTE,
                FASE_SEGUINTE_PROXIMA.NRORDEM
                    || '-'
                    || FASE_SEGUINTE_PROXIMA.NMFASEAPROVSOLI AS NMFASEAPROVSOLI_SEGUINTE,
                CASE
                    WHEN SL.DSPATH IS NOT NULL THEN 'S'
                    WHEN EXISTS (
                        SELECT 1
                        FROM GPE_SOLICITACAODOCUMENTO SLD
                        WHERE SLD.NRORG = SL.NRORG AND SLD.NRSOLICITACAO = SL.NRSOLICITACAO
                    ) THEN 'S'
                    ELSE 'N'
                END ID_EXIST_ARQUIVOS
            FROM
                GPE_SOLICITACAO SL
                LEFT JOIN
                GPE_TIPOSOLICITACAO TP
                    ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_FASETRAMSOLI FT
                    ON FT.NRORG = SL.NRORG AND
                    FT.IDATIVO = 'S'
                AND FT.NRSOLICITACAO = SL.NRSOLICITACAO
                AND FT.NRFASETRAMSOLI = (
                        SELECT
                            MAX(FT1.NRFASETRAMSOLI)
                        FROM
                            GPE_FASETRAMSOLI FT1
                        WHERE
                            FT1.NRORG = FT.NRORG AND
                            FT1.NRSOLICITACAO = FT.NRSOLICITACAO AND
                            FT1.IDATIVO = 'S'
                    )
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_ATUAL
                    ON FASE_ATUAL.NRORG = FT.NRORG
                AND FASE_ATUAL.NRFASEAPROVSOLI = FT.NRFASEAPROVSOLI
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE
                    ON FASE_SEGUINTE.NRORG = SL.NRORG
                AND FASE_SEGUINTE.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                AND FASE_SEGUINTE.NRORDEM = NVL(FASE_ATUAL.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE_PROXIMA
                    ON FASE_SEGUINTE_PROXIMA.NRORG = FASE_SEGUINTE.NRORG
                AND FASE_SEGUINTE_PROXIMA.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                AND FASE_SEGUINTE_PROXIMA.NRORDEM = NVL(FASE_SEGUINTE.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_VINCULOM VM_S
                    ON VM_S.NRORG = SL.NRORG
                AND VM_S.NRVINCULOM = SL.NRVINCULOM
                LEFT JOIN
                GPE_VINCULOH VH_S
                    ON VH_S.NRORG = VM_S.NRORG
                AND VH_S.NRVINCULOM = VM_S.NRVINCULOM
                AND VH_S.DTMESCOMPETENC =
                    (SELECT DTMESCOMPETENC FROM 
                        (SELECT 
                            V1.DTMESCOMPETENC 
                        FROM 
                            GPE_VINCULOH V1 
                        WHERE 
                            V1.DTMESCOMPETENC <= :P_DTMESCOMPETENC 
                            AND V1.NRORG = VH_S.NRORG 
                            AND V1.NRVINCULOM = VH_S.NRVINCULOM 
                            ORDER BY V1.DTMESCOMPETENC DESC) 
                    WHERE ROWNUM = 1)
                LEFT JOIN OPERADOR OP
                    ON OP.CDOPERADOR = SL.CDOPERINCLUSAO
            WHERE
                (SL.NRORG = :P_NRORG OR SL.NRORG = 0)
                AND ( :P_NRPERFAPROVASOLI = 'S' OR
                        (:P_TIPOLEGAL IS NOT NULL AND VH_S.NRESTRUTLEGAL = :P_TIPOLEGAL) OR 
                        (:P_TIPOLEGAL IS NULL AND 
                        (:P_NRESTRUTURAS IS NULL OR (INSTR(:P_NRESTRUTURAS, ',' || VH_S.NRESTRUTGEREN || ',')) > 0))
                )
        ORDER BY NMVINCULOM ASC, SL.DTSOLICITACAO ASC

SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $solicitacoes = $std->fetchAll();

        return $solicitacoes;
    }


    public function getSolicitacoesVinculosGestorByTempLista(
        array $params,
        $nrEstruturas
    ): array {
        $nrorg = $params['P_NRORG'];
        
        $nrLista = NovoCodigo::geraCodigo($nrorg, 'LISTA', 1, 12, true);
        $params['P_NRLISTA'] = $nrLista;
    
        // Monta a query inicial
        $sqlTemp = "INSERT ALL ";
    
        // Acumula os valores para o insert all
        $paramsInsert = [];
        foreach ($nrEstruturas as $index => $nrEstrutura) {
            $sqlTemp .= "INTO LISTA (NRLISTA, CDLISTA) VALUES (:P_NRLISTA_$index, :P_CDLISTA_$index, :P_NRORG_$index) ";
            $paramsInsert["P_NRLISTA_$index"] = $nrLista;
            $paramsInsert["P_CDLISTA_$index"] = $nrEstrutura;
            $paramsInsert["P_NRORG_$index"] = $nrorg;
        }
    
        // Finaliza a query com o SELECT
        $sqlTemp .= "SELECT 1 FROM DUAL";
    
        // Prepara e executa o insert all
        $queryTemp = $this->connection->prepare($sqlTemp);
        $queryTemp->execute($paramsInsert);

        $sql = <<<SQL
            SELECT
                SL.NRORG,
                SL.NRSOLICITACAO,
                SL.NRTIPOSOLICITACAO,
                SL.NRVINCULOM,
                TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY') AS DTSOLICITACAO,
                SL.IDAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOPRI,  'A', 'Aprovada',
                                            'R', 'Reprovada',
                                            'P', 'Pendente',
                                            'E', 'Em andamento',
                                            'Pendente') DSAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOSEC,  'A', 'Aprovada',
                                            'R', 'Reprovada',
                                            'P', 'Pendente',
                                            'E', 'Em andamento',
                                            'Pendente') DSAPROVACAOSEC,
                SL.DTAPROVACAOPRI,
                SL.CDOPERAPROVPRI,
                SL.IDAPROVACAOSEC,
                SL.DTAPROVACAOSEC,
                SL.CDOPERAPROVSEC,
                SL.DSOBSSOLICITACAO,
                SL.DSOBSAPROVAPRI,
                SL.DSOBSAPROVASEC,
                SL.DSPATH,
                VM_S.NMVINCULOM,
                OP.CDOPERADOR,
                OP.NMOPERADOR,
                TP.NMTIPOSOLICITACAO,
                FASE_SEGUINTE.NRFASEAPROVSOLI,
                FASE_SEGUINTE.NRORDEM
                    || '-'
                    || FASE_SEGUINTE.NMFASEAPROVSOLI AS NMFASEAPROVSOLI,
                CASE
                    WHEN :P_NRPERFAPROVASOLI IS NOT NULL AND SL.IDAPROVACAOSEC NOT IN ('A', 'R') THEN 'S'
                    WHEN EXISTS (
                        SELECT 1
                        FROM PERFACESSPARC PERACESS_APROV
                        WHERE PERACESS_APROV.NRORG = SL.NRORG
                            AND INSTR(',' || FASE_SEGUINTE.DSNRPERFILACESSO || ',', ',' || PERACESS_APROV.NRPERFILACESSO ||  ',') > 0
                            AND PERACESS_APROV.NRPARCNEGOCIO = (
                                SELECT NRPARCNEGOCIO
                                FROM OPERADOR
                                WHERE NRORG = PERACESS_APROV.NRORG AND CDOPERADOR = :P_CDOPERADOR_APROVANTE
                            )
                    ) AND SL.IDAPROVACAOSEC NOT IN ('A', 'R') THEN 'S'
                    ELSE 'N'
                END ID_OPER_APROVA_SOLI,
                CASE
                    WHEN FASE_SEGUINTE_PROXIMA.NRFASEAPROVSOLI IS NOT NULL AND SL.IDAPROVACAOSEC NOT IN ('A', 'R') THEN 'S'
                    ELSE 'N'
                END ID_FASE_SEGUINTE_EXISTS,
                FASE_SEGUINTE_PROXIMA.NRFASEAPROVSOLI AS NRFASEAPROVSOLI_SEGUINTE,
                FASE_SEGUINTE_PROXIMA.NRORDEM
                    || '-'
                    || FASE_SEGUINTE_PROXIMA.NMFASEAPROVSOLI AS NMFASEAPROVSOLI_SEGUINTE,
                CASE
                    WHEN SL.DSPATH IS NOT NULL THEN 'S'
                    WHEN EXISTS (
                        SELECT 1
                        FROM GPE_SOLICITACAODOCUMENTO SLD
                        WHERE SLD.NRORG = SL.NRORG AND SLD.NRSOLICITACAO = SL.NRSOLICITACAO
                    ) THEN 'S'
                    ELSE 'N'
                END ID_EXIST_ARQUIVOS
            FROM
                GPE_SOLICITACAO SL
                LEFT JOIN
                GPE_TIPOSOLICITACAO TP
                    ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_FASETRAMSOLI FT
                    ON FT.NRORG = SL.NRORG AND
                    FT.IDATIVO = 'S'
                AND FT.NRSOLICITACAO = SL.NRSOLICITACAO
                AND FT.NRFASETRAMSOLI = (
                        SELECT
                            MAX(FT1.NRFASETRAMSOLI)
                        FROM
                            GPE_FASETRAMSOLI FT1
                        WHERE
                            FT1.NRORG = FT.NRORG AND
                            FT1.NRSOLICITACAO = FT.NRSOLICITACAO AND
                            FT1.IDATIVO = 'S'
                    )
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_ATUAL
                    ON FASE_ATUAL.NRORG = FT.NRORG
                AND FASE_ATUAL.NRFASEAPROVSOLI = FT.NRFASEAPROVSOLI
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE
                    ON FASE_SEGUINTE.NRORG = SL.NRORG
                AND FASE_SEGUINTE.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                AND FASE_SEGUINTE.NRORDEM = NVL(FASE_ATUAL.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE_PROXIMA
                    ON FASE_SEGUINTE_PROXIMA.NRORG = FASE_SEGUINTE.NRORG
                AND FASE_SEGUINTE_PROXIMA.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                AND FASE_SEGUINTE_PROXIMA.NRORDEM = NVL(FASE_SEGUINTE.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_VINCULOM VM_S
                    ON VM_S.NRORG = SL.NRORG
                AND VM_S.NRVINCULOM = SL.NRVINCULOM
                LEFT JOIN
                GPE_VINCULOH VH_S
                    ON VH_S.NRORG = VM_S.NRORG
                AND VH_S.NRVINCULOM = VM_S.NRVINCULOM
                AND VH_S.DTMESCOMPETENC =
                    (SELECT DTMESCOMPETENC FROM 
                        (SELECT 
                            V1.DTMESCOMPETENC 
                        FROM 
                            GPE_VINCULOH V1 
                        WHERE 
                            V1.DTMESCOMPETENC <= :P_DTMESCOMPETENC 
                            AND V1.NRORG = VH_S.NRORG 
                            AND V1.NRVINCULOM = VH_S.NRVINCULOM 
                            ORDER BY V1.DTMESCOMPETENC DESC) 
                    WHERE ROWNUM = 1)
                LEFT JOIN LISTA
                    ON LISTA.NRLISTA = :P_NRLISTA
                LEFT JOIN OPERADOR OP
                    ON OP.CDOPERADOR = SL.CDOPERINCLUSAO
            WHERE
                (SL.NRORG = :P_NRORG OR SL.NRORG = 0)
                AND ( :P_NRPERFAPROVASOLI = 'S' OR
                    (:P_NRLISTA IS NULL OR VH_S.NRESTRUTGEREN = LISTA.CDLISTA))
            ORDER BY NMVINCULOM ASC, SL.DTSOLICITACAO ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $solicitacoes = $std->fetchAll();

        $this->deleteTempLista($nrLista, $nrorg);

        return $solicitacoes;
    }

    public function getItemSolicitacao($nrorg, $nrsolicitacao ) {
        $sql = <<<SQL
    SELECT
        IT.NRORG,
        IT.NRITEMSOLICITACAO,
        IT.NRSOLICITACAO,
        IT.DSCHAVEITEMSOLI,
        IT.DSLABELITEMSOLI,
        IT.DSVALORITEMSOLI,
        IT.DTINCLUSAO,
        IT.DTULTATU,
        IT.NRORGINCLUSAO,
        IT.CDOPERINCLUSAO,
        IT.NRORGULTATU,
        IT.CDOPERULTATU,
        TP.NMTIPOSOLICITACAO
    FROM GPE_ITEMSOLICITACAO IT
    JOIN
          GPE_SOLICITACAO SL
          ON SL.NRSOLICITACAO = IT.NRSOLICITACAO
          AND SL.NRORG = IT.NRORG

    LEFT JOIN
          GPE_TIPOSOLICITACAO TP
          ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO

    WHERE
        IT.NRORG = :P_NRORG
        AND IT.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }


    public function getTiposolicitacao($nrorg, $nrTipoSolicitacao) {
        $sql = <<<SQL
            SELECT * FROM (
                SELECT
                   TS.NRORG,
                   TS.NRTIPOSOLICITACAO,
                   TS.NMTIPOSOLICITACAO,
                   TS.IDATIVO,
                   TS.DTINCLUSAO,
                   TS.DTULTATU,
                   TS.NRORGINCLUSAO,
                   TS.CDOPERINCLUSAO,
                   TS.NRORGULTATU,
                   TS.CDOPERULTATU,
                   NULL NRPERFACESTIPOSOLI,
                   NULL NRMODELOAPROVSOLI,
                   NULL DSCONSIDERACAO
                FROM
                   GPE_TIPOSOLICITACAO TS
                WHERE
                   TS.NRTIPOSOLICITACAO IN (1, 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29)
                  
                /* TIPOS ESPECIFICOS ORG 20 */
                UNION
                SELECT
                   20   AS NRORG,
                   -1   AS NRTIPOSOLICITACAO,
                   'Solicitação ou Cancelamento do Convênio Farmácia' AS NMTIPOSOLICITACAO,
                   'S',
                   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                   'Após o envio da solicitação e assinatura da "Autorização de Desconto de Convênio Farmácia" a Adesão será efetivada no dia 01 do mês subsequente.\n\nApós o envio da solicitação, o Cancelamento será efetivado até o último dia do mês.'
                FROM DUAL
                UNION
                SELECT
                   20   AS NRORG,
                   -2   AS NRTIPOSOLICITACAO,
                   'Solicitação de Nova Via de Cartão' AS NMTIPOSOLICITACAO,
                   'S',
                   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                   'O prazo para entrega de nova via, são 7 dias úteis a partir da solicitação.'
                FROM DUAL
                UNION
                SELECT
                   20   AS NRORG,
                   -3   AS NRTIPOSOLICITACAO,
                   'Solicitação de Alteração do Vale Alimentação ou Refeição' AS NMTIPOSOLICITACAO,
                   'S',
                   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                   'O formulário deve ser enviado até o dia 20 do mês para que a alteração seja efetivada na próxima recarga do benefício.'
                FROM DUAL 
            )
            WHERE (
              NRORG = :P_NRORG
              OR NRORG = 0
            ) AND (NRTIPOSOLICITACAO = :P_NRTIPOSOLICITACAO OR :P_NRTIPOSOLICITACAO IS NULL)
            ORDER BY NMTIPOSOLICITACAO
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'               => $nrorg,
                'P_NRTIPOSOLICITACAO'   => $nrTipoSolicitacao,
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }
    
    public function getSolicitacao($nrorg, $nrsolicitacao = null) {
        $sql = <<<SQL
    SELECT
        SL.*,
        TS.*,
        SL.CDOPERINCLUSAO AS CDOPERINCLUSAOSOLI,
        TS.CDOPERINCLUSAO
    FROM
        GPE_SOLICITACAO SL
    JOIN GPE_TIPOSOLICITACAO TS ON (TS.NRORG = SL.NRORG OR TS.NRORG = 0) AND TS.NRTIPOSOLICITACAO = SL.NRTIPOSOLICITACAO
    WHERE
        SL.NRORG = :P_NRORG
        AND (SL.NRSOLICITACAO = :P_NRSOLICITACAO OR :P_NRSOLICITACAO IS NULL)
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao,
            )
        );

        $resultQuery = empty($nrsolicitacao) ? $query->fetchAll() : $query->fetch();
        
        return $resultQuery;
    }

    public function getTipoRelacionamento(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                TIPORELACIONA
            WHERE
                NRORG = :P_NRORG OR
                NRORG = :P_NRORG_PADRAO
            ORDER BY
                NMTIPORELACIONA ASC
SQL;


        $std = $this->connection->prepare($sql);

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std->execute($params);
        $tiposRel = $std->fetchAll();

        return $tiposRel;
    }

    public function getLinhaTransporteAtiva(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                L.*,
                F.DSFORMATRANSPORTE,
                O.DSOPERTRANSPORTE,
                C.DSCATEGTRANSPORTE,
                T.DSTIPOTRANSPORTE,
                VLT.NRLINHATARIFA,
                VT.VRTARIFA,
                L.NRLINHATRANSP || ' | ' || L.DSLINHATRANSP AS NRDSLINHATRANSP
            FROM
                VAB_LINHA L
                LEFT JOIN
                FORMATRANSPORTE F ON
                    F.NRFORMATRANSPORTE = L.NRFORMATRANSPORTE AND
                    (
                        F.NRORG = L.NRORG OR
                        F.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                OPERTRANSPORTE O ON
                    O.NROPERTRANSPORTE = L.NROPERTRANSPORTE AND
                    (
                        O.NRORG = L.NRORG OR
                        O.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                CATEGTRANSPORTE C ON
                    C.NRCATEGTRANSPORTE = L.NRCATEGTRANSPORTE AND
                    (
                        C.NRORG = L.NRORG OR
                        C.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                TIPOTRANSPORTE T ON
                T.NRTIPOTRANSPORTE = L.NRTIPOTRANSPORTE AND
                (
                    T.NRORG = L.NRORG OR
                    T.NRORG =:P_NRORG_PADRAO
                )
                LEFT JOIN
                VAB_LINHATARIFA VLT ON
                    L.NRLINHATRANSP = VLT.NRLINHATRANSP AND
                    L.NRORG = VLT.NRORG AND
                    (
                        VLT.DTFIMLINHATARIFA > LAST_DAY(:P_DTMESCOMPETENC) OR
                        VLT.DTFIMLINHATARIFA IS NULL
                    )
                    LEFT JOIN
                    VAB_TARIFA VT ON
                    VLT.NRTARIFA = VT.NRTARIFA AND
                    (
                        VT.NRORG = :P_NRORG OR
                        VT.NRORG =:P_NRORG_PADRAO
                    )
            WHERE
                (
                    L.NRORG = :P_NRORG OR
                    L.NRORG =:P_NRORG_PADRAO
                ) AND
                (
                    L.DTFIMLINHA > LAST_DAY(:P_DTMESCOMPETENC) OR
                    L.DTFIMLINHA IS NULL
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $linhas = $std->fetchAll();

        return $linhas;
    }

    public function getTipolinhavinc(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                TIPOLINHAVINC
            WHERE
                NRORG IN (:P_NRORG, :P_NRORGPADRAO) AND
                IDATIVO = 'S'
SQL;

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRORGPADRAO' => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tiposLinhas = $std->fetchAll();

        return $tiposLinhas;
    }

    public function getIntegtransp(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                VAB_INTEGTRANSPM.DTINIVALIDADEINTEG,
                VAB_INTEGTRANSPM.DTFINVALIDADEINTEG,
                VAB_INTEGTRANSPH.*,
                EM.NMESTRUTURAM,
                TIPOLINHAVINC.DSTIPOLINHAVINC,
                TP.DSTIPOINTEGTRANSP
            FROM
                VAB_INTEGTRANSPM
                JOIN
                VAB_INTEGTRANSPH ON
                    VAB_INTEGTRANSPH.NRORG = VAB_INTEGTRANSPM.NRORG AND
                    VAB_INTEGTRANSPH.NRINTEGTRANSPM = VAB_INTEGTRANSPM.NRINTEGTRANSPM
                LEFT JOIN
                TIPOLINHAVINC ON
                    TIPOLINHAVINC.NRTIPOLINHAVINC = VAB_INTEGTRANSPH.NRTIPOLINHAVINC
                LEFT JOIN
                ESTRUTURAM EM ON
                    EM.NRESTRUTURAM = VAB_INTEGTRANSPH.NRESTRUTURAM AND
                    EM.NRORG = VAB_INTEGTRANSPH.NRORG
                LEFT JOIN
                ESTRUTURAH EH ON
                    EH.NRESTRUTURAM = EM.NRESTRUTURAM AND
                    EH.NRORG = EM.NRORG AND
                    EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH
                        WHERE
                            NRORG = EH.NRORG AND
                            NRESTRUTURAM = EH.NRESTRUTURAM AND
                            DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
                LEFT JOIN
                TIPOINTEGTRANSP TP ON
                    (
                        TP.NRORG = VAB_INTEGTRANSPH.NRORG OR
                        TP.NRORG = :P_NRORG_PADRAO
                    ) AND
                    TP.NRTIPOINTEGTRANSP = VAB_INTEGTRANSPH.NRTIPOINTEGTRANSP
            WHERE
                VAB_INTEGTRANSPM.NRORG = :P_NRORG AND
                DTMESCOMPINTEG = (
                    SELECT
                        MAX(DTMESCOMPINTEG)
                    FROM
                        VAB_INTEGTRANSPH H1
                    WHERE
                        H1.NRORG = VAB_INTEGTRANSPH.NRORG AND
                        H1.NRINTEGTRANSPM = VAB_INTEGTRANSPH.NRINTEGTRANSPM AND
                        H1.DTMESCOMPINTEG <= :P_DTMESCOMPETENC
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tiposLinhas = $std->fetchAll();

        return $tiposLinhas;
    }

    public function getSentidolinha(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                SENTIDOLINHA
            WHERE
                NRORG IN (:P_NRORG, :P_NRORG_PADRAO)
SQL;

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $sentidos = $std->fetchAll();

        return $sentidos;
    }
    
    public function getPessoaRelacionamentos(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                PRINCIPAL.*,
                (
                    CASE
                        WHEN NMTIPORELACIONA IS NULL THEN NMPESSOA
                        ELSE (NMPESSOA || ' - ' || NMTIPORELACIONA)
                    END
                ) DSRELACIONA
            FROM (
                SELECT
                    PE.NRPESSOA,
                    PH.NMPESSOA,
                    PH.NMPESSOA AS NMPESSOAPRINC,
                    PH.NRPESSOA AS NRPESSOAPRINC,
                    PH.DTNASCPESSOA AS DTNASCIMENTO,
                    PH.NRCPFPESSOA AS NRCPFPESSOA,
                    PE.NRPARCNEGOCIO,
                    NULL AS NRTIPORELACIONA,
                    '' AS NMTIPORELACIONA
                FROM
                    GPE_PESSOA PE     
                    JOIN
                    GPE_PESSOAH PH ON
                        PH.NRPESSOA = PE.NRPESSOA AND
                        PH.NRORG = PE.NRORG AND
                        PH.DTMESCOMPETENC = (
                            SELECT
                                MAX(DTMESCOMPETENC)                                                  
                            FROM
                                GPE_PESSOAH H1                                                  
                            WHERE
                                H1.NRPESSOA = PH.NRPESSOA AND
                                H1.NRORG = PH.NRORG AND
                                H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )     
                WHERE
                    PE.NRPESSOA = :P_NRPESSOA AND
                    PE.NRORG = :P_NRORG    
                
                UNION
                
                SELECT
                    RELAC.NRPESSOA,
                    RELACH.NMPESSOA,
                    PH.NMPESSOA AS NMPESSOAPRINC,
                    PH.NRPESSOA AS NRPESSOAPRINC,
                    RELACH.DTNASCPESSOA AS DTNASCIMENTO,
                    RELACH.NRCPFPESSOA AS NRCPFPESSOA,
                    RELAC.NRPARCNEGOCIO,
                    TR.NRTIPORELACIONA,
                    TR.NMTIPORELACIONA     
                FROM
                    GPE_PESSOA PRINC     
                    JOIN
                    GPE_PESSOAH PH ON
                        PH.NRPESSOA = PRINC.NRPESSOA AND
                        PH.NRORG = PRINC.NRORG AND
                        PH.DTMESCOMPETENC = (
                            SELECT
                                MAX(DTMESCOMPETENC)                                                  
                            FROM
                                GPE_PESSOAH H1                                                  
                            WHERE
                                H1.NRPESSOA = PH.NRPESSOA AND
                                H1.NRORG = PH.NRORG AND
                                H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )
                    JOIN
                    PARCNEGOCIO PCN ON
                        PCN.NRPARCNEGOCIO = PRINC.NRPARCNEGOCIO AND
                        PCN.NRORG = PRINC.NRORG     
                    JOIN
                    RELACIONAPARC RP ON
                        RP.NRPARCNEGOCIO = PRINC.NRPARCNEGOCIO AND
                        RP.NRORG = PRINC.NRORG     
                    JOIN
                    GPE_PESSOA RELAC ON
                        RELAC.NRPARCNEGOCIO = RP.NRPARCNEGRELAC AND
                        RELAC.NRORG = RP.NRORG     
                    JOIN
                    GPE_PESSOAH RELACH ON
                        RELACH.NRPESSOA = RELAC.NRPESSOA AND
                        RELACH.NRORG = RELAC.NRORG AND
                        RELACH.DTMESCOMPETENC =  (
                            SELECT
                                MAX(DTMESCOMPETENC)              
                            FROM
                                GPE_PESSOAH H1              
                            WHERE
                                H1.NRPESSOA = RELACH.NRPESSOA AND
                                H1.NRORG = RELACH.NRORG AND
                                H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )     
                    JOIN
                    TIPORELACIONA TR ON
                        TR.NRTIPORELACIONA = RP.NRTIPORELACIONA AND
                        (
                            TR.NRORG = RP.NRORG OR
                            TR.NRORG = :P_NRORGPADRAO
                        )     
                WHERE
                    PRINC.NRPESSOA = :P_NRPESSOA AND
                    (
                        INSTR(',' || :P_NRTIPORELACIONA || ',' , ',' || TR.NRTIPORELACIONA || ',') > 0 OR
                        :P_NRTIPORELACIONA IS NULL
                    ) AND
                    PRINC.NRORG = :P_NRORG
                ORDER BY
                    NMPESSOAPRINC,
                    NMPESSOA
            ) PRINCIPAL
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $pessoas = $std->fetchAll();
        
        return $pessoas;
    }

    public function getTipoDependente(
        int $nrOrgPadrao
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                FPA_TIPODEPENDE
            WHERE
                NRORG = :P_NRORG_PADRAO
SQL;

        $params = [
            'P_NRORG_PADRAO' => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tiposDep = $std->fetchAll();

        return $tiposDep;
    }
    
    public function getTemplate(
        int $nrOrg
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                DES_TEMPLATE
            WHERE
                NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
    
    public function getTipoprocessos(
        int $nrOrg
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                DES_TIPOPROCESSO
            WHERE
                NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
    
    public function getParcnegocioVinculos(
        int $nrOrg
    ): array {
        $sql = <<< SQL
            SELECT
                PARC.*,
                OCM.NROCUPACAOM,
                OCH.NMOCUPACAOH
            FROM
                PARCNEGOCIO PARC
                JOIN
                GPE_PESSOA PE ON
                    PE.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO AND
                    PE.NRORG = PARC.NRORG
                JOIN
                GPE_VINCULOM VCM ON
                    PE.NRPESSOA = VCM.NRPESSOA AND
                    PE.NRORG = VCM.NRORG AND
                    VCM.DTRESCISAOVINC IS NULL
                JOIN
                GPE_VINCULOH VCH ON
                    VCM.NRVINCULOM = VCH.NRVINCULOM AND
                    VCM.NRORG = VCH.NRORG AND
                    VCH.DTMESCOMPETENC = (
                        (SELECT DTMESCOMPETENC FROM 
                            (SELECT 
                                H1.DTMESCOMPETENC 
                            FROM 
                                GPE_VINCULOH H1 
                            WHERE 
                                H1.NRORG = VCM.NRORG AND
                                H1.NRVINCULOM = VCM.NRVINCULOM
                            ORDER BY H1.DTMESCOMPETENC DESC) 
                    WHERE ROWNUM = 1)
                    )
                LEFT JOIN
                GPE_OCUPACAOM OCM ON
                    OCM.NROCUPACAOM = NVL(VCH.NRCARGO, VCH.NRFUNCAO) AND
                    OCM.NRORG = VCH.NRORG
                LEFT JOIN
                GPE_OCUPACAOH OCH ON
                    OCM.NROCUPACAOM = OCH.NROCUPACAOM AND
                    OCM.NRORG = OCH.NRORG AND
                    OCH.DTMESCOMPETENC = (
                        (SELECT DTMESCOMPETENC FROM 
                            (SELECT 
                                OCH1.DTMESCOMPETENC 
                            FROM 
                                GPE_OCUPACAOH OCH1 
                            WHERE 
                                OCH1.NRORG = OCM.NRORG AND
                                OCH1.NROCUPACAOM = OCM.NROCUPACAOM 
                            ORDER BY OCH1.DTMESCOMPETENC DESC) 
                        WHERE ROWNUM = 1)
                    )     
            WHERE
                PARC.NRORG = :P_NRORG AND
                PARC.IDATIVO = 'S' AND
                PARC.CDTIPOPARCPRINCIPAL IN ('OPERADOR', 'PESSOA')
            ORDER BY NMPRINCIPALPARC
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getTipoTransferencia(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                GPE_TIPOTRANSFER
            WHERE
                NRORG = :P_NRORG OR
                NRORG = :P_NRORG_PADRAO
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getMovimentacaoTransferencia(
        array $params
    ): array {
        $sql = <<< SQL
            SELECT
                TH.*,TM.DTINIVIGENCIA,
                TM.DTFIMVIGENCIA
            FROM
                GPE_TPMOVTRANSFM TM
                INNER JOIN
                GPE_TPMOVTRANSFH TH ON
                    TM.NRTPMOVTRANSFM = TH.NRTPMOVTRANSFM AND
                    TM.NRORG = TH.NRORG AND
                    TH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_TPMOVTRANSFH H1
                        WHERE
                            H1.NRTPMOVTRANSFM = TH.NRTPMOVTRANSFM AND
                            H1.NRORG = TH.NRORG AND
                            H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
            WHERE
                (
                    TM.NRORG = :P_NRORG OR
                    TM.NRORG= :P_NRORG_PADRAO
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getMotivoTransferencia(
        int $nrOrg
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                GPE_MOTIVOTRANSF
            WHERE
                NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getSolicitacaoByTipoEVinculo(
        int $nrOrg,
        int $nrVinculo,
        int $nrTipoSolicitacao
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                GPE_SOLICITACAO
            WHERE
                NRORG = :P_NRORG AND
                IDATIVO = 'S' AND
                NRVINCULOM = :P_NRVINCULOM AND
                NRTIPOSOLICITACAO = :P_NRTIPOSOLICITACAO AND
                (IDAPROVACAOSEC <> 'R')
SQL;

        $params = [
            'P_NRORG'             => $nrOrg,
            'P_NRVINCULOM'        => $nrVinculo,
            'P_NRTIPOSOLICITACAO' => $nrTipoSolicitacao
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getVinculoSubstituido(
        int $nrOrg,
        int $nrVinculoSubst
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                GPE_VINCULOM GVM
            JOIN GPE_VINCULOH GVH ON GVH.NRORG = :P_NRORG AND GVH.NRVINCULOM = GVM.NRVINCULOM
            WHERE
                GVM.NRORG = :P_NRORG AND
                GVM.NRVINCULOMSUBST = :P_NRVINCULOMSUBST
SQL;

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRVINCULOMSUBST' => $nrVinculoSubst
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getFasesSolicitacao(
        int $nrOrg,
        string $nrsolicitacao
    ): array {
        $sql = <<< SQL
            SELECT
                FS.NRFASEAPROVSOLI,
                FS.NRORDEM,
                FS.NMFASEAPROVSOLI,
                NVL(FT.IDAPROVADO, 'P') IDAPROVADO,
                DECODE (NVL(FT.IDAPROVADO, 'P'),    'S', 'Aprovada',
                                                    'N', 'Reprovada',
                                                    'P', 'Pendente',
                                                    'Pendente') DSAPROVADO,
                FT.CDOPERINCLUSAO,
                OP.NMOPERADOR,
                FT.DTINCLUSAO DTINCLUSAO,
                FT.DSOBSERVACAO
            FROM
                GPE_SOLICITACAO SL
                JOIN GPE_FASEAPROVSOLI    FS ON FS.NRORG = SL.NRORG
                                            AND FS.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                LEFT JOIN GPE_FASETRAMSOLI     FT ON FT.NRORG = FS.NRORG
                                                AND FT.NRFASEAPROVSOLI = FS.NRFASEAPROVSOLI
                                                AND FT.NRSOLICITACAO = SL.NRSOLICITACAO
                                                AND FT.IDATIVO = 'S'
                LEFT JOIN OPERADOR             OP ON OP.CDOPERADOR = FT.CDOPERINCLUSAO
                                                AND OP.NRORG = SL.NRORG
            WHERE
                    SL.NRORG = :P_NRORG
                AND SL.NRSOLICITACAO = :P_NRSOLICITACAO
            ORDER BY FS.NRORDEM
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRSOLICITACAO' => $nrsolicitacao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getFolhaFechada(
        int $nrOrg,
        int $nrVinculom,
        string $dtAlteracao
    ): array {
        $sql = <<< SQL
            SELECT *
            FROM FPA_CALCULOFOLHA
            WHERE NRORG = :P_NRORG
            AND TRUNC(DTOCORRENCIA, 'MM') = :P_DTALTERACAO
            AND NRTIPOMOVIMENT = 1
            AND NRTPMODALIDCAL = 1
            AND NROCORRECAL    = 1
            AND EXISTS (SELECT 1
                            FROM FPA_ITECALCFOLHA
                            WHERE NRORG = :P_NRORG
                            AND NRCALCULOFOLHA = FPA_CALCULOFOLHA.NRCALCULOFOLHA
                            AND NRVINCULOM = :P_NRVINCULOM
                            AND NREVENTOM IN (3030, 3050))
SQL;

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRVINCULOM'  => $nrVinculom,
            'P_DTALTERACAO' => $dtAlteracao
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getHierarquiaByEstruturaLegalAndGeren(
        int $nrOrg,
        int $nrEstrutLegal,
        int $nrEstrutGeren,
        string $dtAtual
    ): array {
        $sql = <<< SQL
            SELECT * 
                FROM HIERARQUIAORG 
            WHERE NRORG = :P_NRORG
                AND NRESTRUTURAMSUP = :P_NRESTRUTLEGAL
                AND NRESTRUTURAMREL = :P_NRESTRUTGEREN
                AND (DTFIMVIGENCIA >= :P_DTATUAL OR DTFIMVIGENCIA IS NULL)
                AND IDATIVO = 'S'
SQL;

        $params = [
            'P_NRORG'         => $nrOrg,
            'P_NRESTRUTLEGAL' => $nrEstrutLegal,
            'P_NRESTRUTGEREN' => $nrEstrutGeren,
            'P_DTATUAL'       => $dtAtual
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getSolicitacoesAdmissao(
        int $nrOrg
    ): array {
        $sql = <<< SQL
            SELECT
                SL.NRSOLICITACAO,
                TS.NMTIPOSOLICITACAO,
                SL.NRSOLICITACAO || ' | (' ||  DSP.NMPROCESSO  || ') - ' || TS.NMTIPOSOLICITACAO AS NRNMSOLICITACAO,
                SL.NRVINCULOM,
                SL.IDAPROVACAOSEC,
                SL.NRTIPOSOLICITACAO,
                DSP.NRPROCESSO,
                DSP.NMPROCESSO
            FROM
                GPE_SOLICITACAO SL
                JOIN GPE_TIPOSOLICITACAO TS ON (TS.NRORG = SL.NRORG OR TS.NRORG = 0) AND TS.NRTIPOSOLICITACAO = SL.NRTIPOSOLICITACAO
                JOIN GPE_ITEMSOLICITACAO ISL ON ISL.NRORG = SL.NRORG AND ISL.NRSOLICITACAO = SL.NRSOLICITACAO AND ISL.DSCHAVEITEMSOLI = 'NRPROCESSO'
                JOIN DES_PROCESSO DSP ON DSP.NRORG = SL.NRORG AND DSP.NRPROCESSO = ISL.DSVALORITEMSOLI AND DSP.NRVAGAS > DSP.NRVAGASADMITIDAS
            WHERE
                SL.NRORG = :P_NRORG AND
                SL.IDATIVO = 'S' AND
                SL.NRTIPOSOLICITACAO IN (25, 28) AND
                SL.IDAPROVACAOSEC = 'A'
            ORDER BY SL.NRSOLICITACAO DESC
SQL;

        $params = [
            'P_NRORG'             => $nrOrg
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
        

    public function deleteItemSolicitacao($nrsolicitacao, $nrOrg) {
        $sql = <<<SQL
            DELETE FROM
                GPE_ITEMSOLICITACAO
            WHERE 
                NRORG = :P_NRORG
                AND NRSOLICITACAO = :P_NRSOLICITACAO
SQL;

        $params = [
            'P_NRORG'         => $nrOrg,
            'P_NRSOLICITACAO' => $nrsolicitacao
        ];

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function resetFases($params) {
        $sql = <<<SQL
        UPDATE
            GPE_FASETRAMSOLI
        SET
            IDATIVO = 'N'
        WHERE 
            NRORG = :P_NRORG
            AND NRSOLICITACAO = :P_NRSOLICITACAO
  
SQL;
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
    }

    public function resetStatus($params) {
        $sql = <<<SQL
        UPDATE
            GPE_SOLICITACAO
        SET
            IDAPROVACAOSEC = 'P'
        WHERE 
            NRORG = :P_NRORG
            AND NRSOLICITACAO = :P_NRSOLICITACAO
  
SQL;
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
    }
    
    public function getMotivoRecusa($params) {
        $sql = <<<SQL
        SELECT
            FS.NRFASEAPROVSOLI,
            FS.NMFASEAPROVSOLI,
            FT.CDOPERINCLUSAO,
            OP.NMOPERADOR,
            FT.DTINCLUSAO DTINCLUSAO,
            FT.DSOBSERVACAO
        FROM
            GPE_SOLICITACAO SL
            JOIN GPE_FASEAPROVSOLI    FS ON FS.NRORG = SL.NRORG
                                        AND FS.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
            LEFT JOIN GPE_FASETRAMSOLI     FT ON FT.NRORG = FS.NRORG
                                            AND FT.NRFASEAPROVSOLI = FS.NRFASEAPROVSOLI
                                            AND FT.NRSOLICITACAO = SL.NRSOLICITACAO
                                            AND FT.IDAPROVADO = 'N'
            LEFT JOIN OPERADOR             OP ON OP.CDOPERADOR = FT.CDOPERINCLUSAO
                                            AND OP.NRORG = SL.NRORG
        WHERE
                SL.NRORG = :P_NRORG
                AND SL.NRSOLICITACAO = :P_NRSOLICITACAO
                AND FT.IDAPROVADO = 'N'
        ORDER BY FT.NRFASETRAMSOLI DESC
  
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function updateAnexo($params) {
        $sql = <<<SQL
        UPDATE
            GPE_SOLICITACAO
        SET
            DSPATH = :P_DSPATH
        WHERE 
            NRORG = :P_NRORG
            AND NRSOLICITACAO = :P_NRSOLICITACAO
  
SQL;
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
    }

    public function verificaPerfilAcessoSST(string $nrorg, string $cdoperador) {
        try {
            $sql = <<<SQL
                SELECT 
                    1
                FROM 
                    OPERADOR OP
                    JOIN PERFACESSPARC PA
                        ON PA.NRORG = OP.NRORG
                            AND PA.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                    LEFT JOIN GPE_PARAMORG PO
                        ON PO.NRORG = OP.NRORG
                WHERE 
                    OP.NRORG = :P_NRORG
                    AND OP.CDOPERADOR = :P_CDOPERADOR
                    AND PO.NRPERFAPROVASOLI = PA.NRPERFILACESSO
SQL;
            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"       => $nrorg,
                    "P_CDOPERADOR"  => $cdoperador
                )
            );

            $resultQuery = $query->fetchAll();

            if (!empty($resultQuery)) {
                return 'S';
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDocumentosSolicitacao($nrorg, $nrsolicitacao = null) {
        $sql = <<<SQL
    SELECT
        SLD.NRSOLICITACAO,
        SLD.DSPATH,
        SLD.NRORG,
        SLD.NMFILE,
        SLD.IDTYPE,
        SLD.DTINCLUSAO,
        SLD.CDOPERINCLUSAO,
        COALESCE(OP.NMOPERADOR, 'Não Definido') AS NMOPERADOR
    FROM
        GPE_SOLICITACAODOCUMENTO SLD
    LEFT JOIN OPERADOR OP ON OP.CDOPERADOR = SLD.CDOPERINCLUSAO AND OP.NRORG = SLD.NRORG
    WHERE
        SLD.NRORG = :P_NRORG AND SLD.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao,
            )
        );

        $result = $query->fetchAll();
        
        return !empty($result) ? $result : [];
    }

    public function getDocumentoSolicitacao($nrorg, $nrsolicitacao = null) {
        $sql = <<<SQL
    SELECT
        SL.NRSOLICITACAO,
        SL.DSPATH,
        SL.NRORG,
        NULL AS NMFILE,
        NULL AS IDTYPE,
        SL.DTINCLUSAO,
        SL.CDOPERINCLUSAO,
        OP.NMOPERADOR
    FROM
        GPE_SOLICITACAO SL
    LEFT JOIN OPERADOR OP ON OP.CDOPERADOR = SL.CDOPERINCLUSAO AND OP.NRORG = SL.NRORG
    WHERE
        SL.NRORG = :P_NRORG AND SL.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao,
            )
        );

        $result = $query->fetchAll();
        
        return !empty($result) ? $result : [];
    }

    public function setDocumentoSolicitacao(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO GPE_SOLICITACAODOCUMENTO (
                NRORG,              NRSOLICITACAODOCUMENTO,      NRSOLICITACAO,
                DSPATH,             DSOBSSOLICITACAODOCUMENTO,   NMFILE,
                IDTYPE,             IDATIVO,                     NRORGINCLUSAO,
                DTINCLUSAO,         CDOPERINCLUSAO,              NRORGULTATU,
                DTULTATU,           CDOPERULTATU
            )
            VALUES (
                :NRORG,              :NRSOLICITACAODOCUMENTO,      :NRSOLICITACAO,
                :DSPATH,             :DSOBSSOLICITACAODOCUMENTO,   :NMFILE,
                :IDTYPE,             :IDATIVO,                     :NRORGINCLUSAO,
                :DTINCLUSAO,         :CDOPERINCLUSAO,              :NRORGULTATU,
                :DTULTATU,           :CDOPERULTATU
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getInfoFase(
        int $nrOrg,
        string $nrsolicitacao
    ): array {

        $sql = <<<SQL
    SELECT
        S.NRORG,
        S.NRSOLICITACAO,
        S.NRTIPOSOLICITACAO,
        S.NRMODELOAPROVSOLI,
        TS.NMTIPOSOLICITACAO,
        S.NRVINCULOM,
        VM.NMVINCULOM,
        FASE_TRAM_ATUAL.NRFASETRAMSOLI NRFASETRAMSOLI_ATUAL,
        FASE_TRAM_ATUAL.IDAPROVADO IDAPROVADO_ATUAL,
        CASE
            WHEN FASE_SEGUINTE.NRFASEAPROVSOLI IS NOT NULL
                AND (FASE_TRAM_ATUAL.IDAPROVADO = 'S' OR (FASE_TRAM_ATUAL.IDAPROVADO = 'N' AND FASE_ATUAL.IDFASEOBRIGATORIA = 'N') OR FASE_TRAM_ATUAL.NRFASETRAMSOLI IS NULL)
                AND S.IDAPROVACAOSEC NOT IN ('R', 'A') THEN 'S'
            ELSE 'N'
        END FASE_ATUAL_VALIDA_APROVACAO,
        FASE_TRAM_ATUAL.DSOBSERVACAO DSOBSERVACAO_ATUAL,
        FASE_SEGUINTE.NRFASEAPROVSOLI NRFASEAPROVSOLI_SEGUINTE,
        FASE_SEGUINTE.NRORDEM NRORDEM_SEGUINTE,
        FASE_SEGUINTE.IDFASEOBRIGATORIA IDFASEOBRIGATORIA_SEGUINTE,
        CASE
            WHEN FASE_SEGUINTE.NRFASEAPROVSOLI = ULTIMA_FASE.NRFASEAPROVSOLI THEN 'S'
            ELSE 'N'
        END FASE_ATUAL_EH_ULTIMA
    FROM GPE_SOLICITACAO S
    JOIN GPE_VINCULOM VM ON VM.NRORG = S.NRORG
        AND VM.NRVINCULOM = S.NRVINCULOM
    JOIN GPE_TIPOSOLICITACAO TS ON TS.NRTIPOSOLICITACAO = S.NRTIPOSOLICITACAO
    JOIN GPE_MODELOAPROVSOLI M ON M.NRORG = S.NRORG
        AND M.NRMODELOAPROVSOLI = S.NRMODELOAPROVSOLI
    LEFT JOIN GPE_FASETRAMSOLI FASE_TRAM_ATUAL ON FASE_TRAM_ATUAL.NRORG = S.NRORG
        AND FASE_TRAM_ATUAL.NRSOLICITACAO = S.NRSOLICITACAO
        AND FASE_TRAM_ATUAL.IDATIVO = 'S'
        AND FASE_TRAM_ATUAL.NRFASEAPROVSOLI = (
            SELECT F2.NRFASEAPROVSOLI
            FROM GPE_FASETRAMSOLI F2
            LEFT JOIN GPE_FASEAPROVSOLI FA2 ON FA2.NRORG = F2.NRORG
                AND FA2.NRFASEAPROVSOLI = F2.NRFASEAPROVSOLI
            WHERE F2.NRORG = FASE_TRAM_ATUAL.NRORG
                AND F2.NRSOLICITACAO = FASE_TRAM_ATUAL.NRSOLICITACAO
                AND F2.IDATIVO = 'S'
            ORDER BY FA2.NRORDEM DESC
            FETCH FIRST ROW ONLY
        )
    LEFT JOIN GPE_FASEAPROVSOLI FASE_ATUAL ON FASE_ATUAL.NRORG = S.NRORG
        AND FASE_ATUAL.NRFASEAPROVSOLI = FASE_TRAM_ATUAL.NRFASEAPROVSOLI
    LEFT JOIN GPE_FASEAPROVSOLI FASE_SEGUINTE ON FASE_SEGUINTE.NRORG = S.NRORG
        AND FASE_SEGUINTE.NRMODELOAPROVSOLI = S.NRMODELOAPROVSOLI
        AND FASE_SEGUINTE.NRORDEM = (
            SELECT MIN(NRORDEM)
            FROM GPE_FASEAPROVSOLI
            WHERE NRORG = FASE_SEGUINTE.NRORG
                AND NRMODELOAPROVSOLI = FASE_SEGUINTE.NRMODELOAPROVSOLI
                AND NRFASEAPROVSOLI > NVL(FASE_TRAM_ATUAL.NRFASEAPROVSOLI, -1)
        )
    LEFT JOIN GPE_FASEAPROVSOLI ULTIMA_FASE ON ULTIMA_FASE.NRORG = S.NRORG
        AND ULTIMA_FASE.NRMODELOAPROVSOLI = S.NRMODELOAPROVSOLI
        AND ULTIMA_FASE.NRORDEM = (
            SELECT MAX(NRORDEM)
            FROM GPE_FASEAPROVSOLI
            WHERE NRORG = ULTIMA_FASE.NRORG
                AND NRMODELOAPROVSOLI = ULTIMA_FASE.NRMODELOAPROVSOLI
    )
    WHERE S.NRORG = :P_NRORG
        AND S.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRSOLICITACAO' => $nrsolicitacao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function getInfoFaseWithEmails(
        int $nrOrg,
        string $nrsolicitacao
    ): array {

        $sql = <<<SQL
    SELECT
        S.NRORG,
        S.NRSOLICITACAO,
        S.NRTIPOSOLICITACAO,
        S.NRMODELOAPROVSOLI,
        TS.NMTIPOSOLICITACAO,
        S.NRVINCULOM,
        VM.NMVINCULOM,
        FASE_TRAM_ATUAL.NRFASETRAMSOLI NRFASETRAMSOLI_ATUAL,
        FASE_TRAM_ATUAL.IDAPROVADO IDAPROVADO_ATUAL,
        CASE
            WHEN FASE_SEGUINTE.NRFASEAPROVSOLI IS NOT NULL
                AND (FASE_TRAM_ATUAL.IDAPROVADO = 'S' OR (FASE_TRAM_ATUAL.IDAPROVADO = 'N' AND FASE_ATUAL.IDFASEOBRIGATORIA = 'N') OR FASE_TRAM_ATUAL.NRFASETRAMSOLI IS NULL)
                AND S.IDAPROVACAOSEC NOT IN ('R', 'A') THEN 'S'
            ELSE 'N'
        END FASE_ATUAL_VALIDA_APROVACAO,
        FASE_TRAM_ATUAL.DSOBSERVACAO DSOBSERVACAO_ATUAL,
        FASE_SEGUINTE.NRFASEAPROVSOLI NRFASEAPROVSOLI_SEGUINTE,
        FASE_SEGUINTE.NRORDEM NRORDEM_SEGUINTE,
        FASE_SEGUINTE.IDFASEOBRIGATORIA IDFASEOBRIGATORIA_SEGUINTE,
        CASE
            WHEN FASE_SEGUINTE.NRFASEAPROVSOLI = ULTIMA_FASE.NRFASEAPROVSOLI THEN 'S'
            ELSE 'N'
        END FASE_ATUAL_EH_ULTIMA,
        EMAILS_APROV_FASE_ANTERIOR.DSEMAILSOPER DSEMAILSOPER_FASE_ANTERIOR
    FROM GPE_SOLICITACAO S
    JOIN GPE_VINCULOM VM ON VM.NRORG = S.NRORG
        AND VM.NRVINCULOM = S.NRVINCULOM
    JOIN GPE_TIPOSOLICITACAO TS ON TS.NRTIPOSOLICITACAO = S.NRTIPOSOLICITACAO
    JOIN GPE_MODELOAPROVSOLI M ON M.NRORG = S.NRORG
        AND M.NRMODELOAPROVSOLI = S.NRMODELOAPROVSOLI
    LEFT JOIN GPE_FASETRAMSOLI FASE_TRAM_ATUAL ON FASE_TRAM_ATUAL.NRORG = S.NRORG
        AND FASE_TRAM_ATUAL.NRSOLICITACAO = S.NRSOLICITACAO
        AND FASE_TRAM_ATUAL.IDATIVO = 'S'
        AND FASE_TRAM_ATUAL.NRFASEAPROVSOLI = (
            SELECT F2.NRFASEAPROVSOLI
            FROM GPE_FASETRAMSOLI F2
            LEFT JOIN GPE_FASEAPROVSOLI FA2 ON FA2.NRORG = F2.NRORG
                AND FA2.NRFASEAPROVSOLI = F2.NRFASEAPROVSOLI
            WHERE F2.NRORG = FASE_TRAM_ATUAL.NRORG
                AND F2.NRSOLICITACAO = FASE_TRAM_ATUAL.NRSOLICITACAO
                AND F2.IDATIVO = 'S'
            ORDER BY FA2.NRORDEM DESC
            FETCH FIRST ROW ONLY
        )
    LEFT JOIN GPE_FASEAPROVSOLI FASE_ATUAL ON FASE_ATUAL.NRORG = S.NRORG
        AND FASE_ATUAL.NRFASEAPROVSOLI = FASE_TRAM_ATUAL.NRFASEAPROVSOLI
    LEFT JOIN GPE_FASEAPROVSOLI FASE_SEGUINTE ON FASE_SEGUINTE.NRORG = S.NRORG
        AND FASE_SEGUINTE.NRMODELOAPROVSOLI = S.NRMODELOAPROVSOLI
        AND FASE_SEGUINTE.NRORDEM = (
            SELECT MIN(NRORDEM)
            FROM GPE_FASEAPROVSOLI
            WHERE NRORG = FASE_SEGUINTE.NRORG
                AND NRMODELOAPROVSOLI = FASE_SEGUINTE.NRMODELOAPROVSOLI
                AND NRFASEAPROVSOLI > NVL(FASE_TRAM_ATUAL.NRFASEAPROVSOLI, -1)
        )
    LEFT JOIN GPE_FASEAPROVSOLI ULTIMA_FASE ON ULTIMA_FASE.NRORG = S.NRORG
        AND ULTIMA_FASE.NRMODELOAPROVSOLI = S.NRMODELOAPROVSOLI
        AND ULTIMA_FASE.NRORDEM = (
            SELECT MAX(NRORDEM)
            FROM GPE_FASEAPROVSOLI
            WHERE NRORG = ULTIMA_FASE.NRORG
                AND NRMODELOAPROVSOLI = ULTIMA_FASE.NRMODELOAPROVSOLI
    )
    LEFT JOIN (
        SELECT
            FTS.NRSOLICITACAO,
            LISTAGG(DISTINCT EMAILS.DSEMAILOPER, ',') WITHIN GROUP (ORDER BY EMAILS.DSEMAILOPER) AS DSEMAILSOPER
        FROM GPE_FASETRAMSOLI FTS
        JOIN (
            SELECT DISTINCT OPP.DSEMAILOPER, FTS1.NRSOLICITACAO
            FROM GPE_FASETRAMSOLI FTS1
            JOIN OPERADOR OPP ON OPP.NRORG = FTS1.NRORG
                AND OPP.CDOPERADOR = FTS1.CDOPERINCLUSAO
            WHERE FTS1.NRORG = :P_NRORG
            UNION
            SELECT DISTINCT OP.DSEMAILOPER, S1.NRSOLICITACAO
            FROM GPE_SOLICITACAO S1
            JOIN OPERADOR OP ON OP.NRORG = S1.NRORG
                AND OP.CDOPERADOR = S1.CDOPERINCLUSAO
            WHERE S1.NRORG = :P_NRORG
                AND S1.NRSOLICITACAO = :P_NRSOLICITACAO
        ) EMAILS ON EMAILS.NRSOLICITACAO = FTS.NRSOLICITACAO
        WHERE FTS.NRORG = :P_NRORG
        AND FTS.NRSOLICITACAO = :P_NRSOLICITACAO
        GROUP BY FTS.NRSOLICITACAO
    ) EMAILS_APROV_FASE_ANTERIOR ON EMAILS_APROV_FASE_ANTERIOR.NRSOLICITACAO = S.NRSOLICITACAO
    WHERE S.NRORG = :P_NRORG
        AND S.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRSOLICITACAO' => $nrsolicitacao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function deleteTempLista($nrLista, $nrorg) {
        $sql = <<<SQL
            DELETE FROM
                LISTA
            WHERE 
                NRLISTA = :P_NRLISTA AND NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRLISTA' => $nrLista,
            'P_NRORG'   => $nrorg 
        ];

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
}