<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 08/08/2016
 * Time: 14:17
 */

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\SituacaoFuncionalService;
use HCM\Geral\Services\CrudService;

class SituacaoFuncionalController extends Crud{

    protected $dataSourceName = 'gpe_situfunc';
    protected $dataSourceName_M = 'gpe_situfuncm';
    protected $dataSourceName_H = 'gpe_situfunch';
    protected $dataSourceName_get = 'gpe_situfunc_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTEFETIVA', 'P_ATIVO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, CrudService $crudService, SituacaoFuncionalService $situacaoFuncionalService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->crudService = $crudService;
        $this->situacaoFuncionalService = $situacaoFuncionalService;
        parent::__construct($dataSourceManager);
    }

    public function getSitufunc(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setSitufunc (DTO\Request\DataSet $request, DTO\Response $response) {
        
        try{
            $this->dataSourceName = $this->dataSourceName_M;
            
            foreach($request->getDataSet()->getRows() as $row) {
                
                if (!array_key_exists('NRSITUFREQUENCIA', $row)) {
                    $row['NRSITUFREQUENCIA'] = '';
                }
                if (!array_key_exists('NRPROXSITUFUNC', $row)) {
                    $row['NRPROXSITUFUNC'] = '';
                }
            }
            
    			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
    			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
    			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
    			$this->crudService->save($this->dataSourceName, $rows);
    			$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
    			
    			$currentRow = current($request->getDataSet()->getRows());
            if(isset($currentRow['saveHistoryTable'])) {
                    
                    if (!$currentRow['__is_new']) {
                        if (!$this->checkHistory($response, $currentRow)) {
                            // is new history
                            $currentRow['__is_new'] = true;
                        }
                    }
                    
    				$tableMasterPK = strtoupper($currentRow['saveHistoryTable']['__table_master_pk']);
    				$newCode = array_filter($this->crudService->getColumnsToReturn(), function ($element) use ($tableMasterPK) {
    					return $element[$tableMasterPK];
    				});
    
    				$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($currentRow['saveHistoryTable']['__table_history_name']);
    				$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
    				$currentRow[strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])] = intval($newCode[0][strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])]);
    
                    // set history properties
                    $currentRow['__new_code_pk'] = $currentRow['saveHistoryTable']['__table_history_name'];
                    $currentRow['__table_pk'] = $currentRow['saveHistoryTable']['__table_history_pk'];
    
    	        	$newDataSet = new DataSet($this->dataSourceName, array($currentRow));
    	        	$newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet,
    	            $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    				$rows = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
    				$this->crudService->save($this->dataSourceName, $rows);
                }
            $response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }

    public function deleteSitufunc(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->situacaoFuncionalService->deleteSituacao($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
    public function checkHistory($response, $row) {
        $condition = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->hcmParameters->getNrorgTrabalho()
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_master_pk']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_master_pk'])]
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_competenc']),
                "operator" => "=",
                "value" => $this->hcmParameters->getDtCompetencia()
            )
        );
        
        $filterCriteria = new FilterCriteria($row['saveHistoryTable']['__table_history_name'], $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array());
       
        $this->crudService->find($response, $newFilterCriteria);
        if (empty($response->getDataSets()[0]->getRows())) 
            return false;
        else 
            return true;
        
    }
    
}